
// Name to be used by the script when creating firewalls, and relationships.

var firewallName = "VPN Adapter Firewall";
var firewallLevel = 101;  // level, used for conflict resolution


// identify the adapter(s), this firewall should be applied too.
//   array is:
//        friendly Name,      -- can be empty("") or part of the friendly name, match is partial and case insensitive
//        Mac Address,        -- either empty("") or "01:02:03:04:05:06"
//        type                -- valid values "any", "wired", "wireless", empty("")

var adapterInformation = Array(
    new Array("Nortel", "", "any"),
    new Array("Cisco AnyConnect Secure Mobility Client", "", "any"),
    new Array("Juniper Network Connect Virtual Adapter", "", "any"),
    new Array("TAP-Win32 Adapter", "", "any")
);

// The following variables are used to configure the firewall to be applied to the the above listed adapters
var defaultFWBehavior = "stateful";

// standard controls to be enabled
var standardControls = new Array("802.1x", "arp", "icmp", "ip_multicast", "ip_subnet", "multicast", "snap");

// standard port rules to be set
var standardPortRulesOpen = new Array();            // new Array("802.1x", "arp", "icmp");
var standardPortRulesClosed = new Array();          // new Array("802.1x", "arp", "icmp");

// standard access control rules to be added to firewall
var standardACLRulesTrusted = new Array();          // new Array("dhcp", "dns", "gateway", "zenworks");
var standardACLRulesNotTrusted = new Array();       // new Array("dhcp", "dns", "gateway", "zenworks");

// Port Rules to be applied       // sample: new Array("type", "behavior", "start", "end"), 
var portRules = new Array(
    // UDP Ephemeral Ports
    new Array("udp", "open", "1024", "65535"),
    new Array("tcp/udp", "open", "8080", "8081"),

    //Ports for SCCM
    new Array("udp", "open", "67", "69"),
    new Array("tcp/udp", "open", "80", "80"),
    new Array("tcp/udp", "open", "135", "135"),
    new Array("tcp/udp", "open", "137", "139"),
    new Array("tcp/udp", "open", "389", "389"),
    new Array("tcp/udp", "open", "443", "443"),
    new Array("tcp/udp", "open", "445", "445"),
    new Array("tcp/udp", "open", "636", "636"),
    new Array("tcp/udp", "open", "1433", "1433"),
    new Array("tcp/udp", "open", "1723", "1723"),
    new Array("tcp/udp", "open", "2701", "2704"),
    new Array("tcp/udp", "open", "3268", "3269"),
    new Array("tcp/udp", "open", "3389", "3389"),
    new Array("tcp/udp", "open", "8530", "8531"),
    new Array("tcp/udp", "open", "9738", "9738"),

// Provide Remote Desktop Access from any host
    new Array("tcp/udp", "open", "3389", "3389")
    );

// Access controls to be applied.
//   Each array must contain 3 elements - 
//            type - macro, ip, mac
//            behavior - trusted, not_trusted
//            value -
//                if type = macor, than dns, dhcp, gateway, wins, zenworks, dns_all, dhcp_all, gateway_all, wins_all
//                if type = ip, than  ip address, ip range, or cidr notation
//                if type = mac than physical address (current or permanent) of the form: 00:11:22:33:44:55
var aclRules = new Array(
    new Array("macro", "trusted", "dhcp"),
    new Array("ip", "trusted", "192.168.0.0/24") 
);


// Main Routine
var uniqueId = GenerateUniqueId();
DisplayVersionInfo();
DisplayThisScriptInfo();
DisplayTriggerParameters();

var ret = ValidateScriptingVersion();
if (ret == 0) {

    ret = ScriptAlreadyRun();
    if (ret == 0) {

        CreateFirewall();
        CreateFirewallRelations();
    }
    else {
        Action.Trace("Firewall already exists: " + uniqueId);
    }
    var firewall = Query.GetFirewall(uniqueId);
    DisplayFirewall(firewall);
    DisplayFirewallRelationRules();
    DisplayFirewallAdapterInfo();
}
else {
    Action.Trace("ZENWorks Endpoint Security does not support the required api version");
}


function GenerateUniqueId() {
    // using the script id and version combined will provide a unique identifier
    // allow the script to check if has already been run previously because 
    // the firewall can be detected. If the script is changed, at a minimum the policy
    // must be given a new version.
    return Query.ScriptID + "-" + Query.ScriptVersion;
}


function ValidateScriptingVersion() {
    // validates that the client recieving this policy supports the scripting api required.
    // Api for firewall requires a version of 11.1.0.0 or higher.
    if (Query.Version("Action", "Major") >= 11
        && Query.Version("Action", "Minor") >= 1)
        return 0;
    return 1;
}

function ScriptAlreadyRun() {
    // Incase the policy is set up to trigger more often than required, this routine checks
    // if the firewall has already been defined.

    var firewall = Query.GetFirewall(uniqueId);
    if (firewall != null)
        return 1;
    return 0;
}

function CreateFirewall() {
    Action.Trace("******** Create firewall ******");
    var ret = Action.CreateFWSandbox(defaultFWBehavior);
    if (ret != 0)
        Action.Trace("Create FW Sandbox failed: " + ret);

    for (i = 0; i < standardControls.length; i++) {
        ret = Action.AddStandardControlToFWSandbox(standardControls[i], "enabled");
        if (ret != 0)
            Action.Trace("Add Standard Control " + standardControls[i] + " To Sandbox failed to enable: " + ret);
    }

    for (i = 0; i < standardPortRulesOpen.length; i++) {
        ret = Action.AddStandardPortRuleToFWSandbox(standardPortRulesOpen[i], "open");
        if (ret != 0)
            Action.Trace("Add Standard Port Rule (open) " + standardPortRulesOpen[i] + " To Sandbox failed: " + ret);
    }

    for (i = 0; i < standardPortRulesClosed.length; i++) {
        ret = Action.AddStandardPortRuleToFWSandbox(standardPortRulesClosed[i], "closed");
        if (ret != 0)
            Action.Trace("Add Standard Port Rule (closed) " + standardPortRulesClosed[i] + " To Sandbox failed: " + ret);
    }

    for (i = 0; i < standardACLRulesTrusted.length; i++) {
        ret = Action.AddStandardAclToFWSandbox(standardACLRulesTrusted[i], "trusted");
        if (ret != 0)
            Action.Trace("Add Standard Acl (trusted) " + standardACLRulesTrusted[i] + " To Sandbox failed: " + ret);
    }

    for (i = 0; i < standardACLRulesNotTrusted.length; i++) {
        ret = Action.AddStandardAclToFWSandbox(standardACLRulesNotTrusted[i], "not_trusted");
        if (ret != 0)
            Action.Trace("Add Standard Acl (not_trusted) " + standardACLRulesNotTrusted[i] + " To Sandbox failed: " + ret);
    }

    for (i = 0; i < portRules.length; i++) {
        if (portRules[i].length == 4)
        {
           ret = Action.AddPortRuleToFWSandbox(portRules[i][0], portRules[i][1], portRules[i][2], portRules[i][3]);
            if (ret != 0)
                Action.Trace("Add Port Rule: type: " + portRules[i][0] + " behavior: " + portRules[i][1] + " start: " + portRules[i][2] + " end: " + portRules[i][3] + " To Sandbox failed: " + ret);
        }
        else
        {
            Action.Trace("Port Rule: " + i + " , is invalid");
        }
    }

    for (i = 0; i < aclRules.length; i++) {
        if (aclRules[i].length == 3)
        {
           ret = Action.AddAclToFWSandbox(aclRules[i][0], aclRules[i][1], aclRules[i][2]);
            if (ret != 0)
                Action.Trace("Add Access Control: type: " + aclRules[i][0] + " behavior: " + aclRules[i][1] + " value: " + aclRules[i][2] + " To Sandbox failed: " + ret);
        }
        else
        {
            Action.Trace("Acl: " + i + " , is invalid");
        }
    }

    ret = Action.PublishFWSandbox(uniqueId, firewallName);
    if (ret != 0) {
        Action.Trace("Publish Sandbox failed: " + ret);
    }
    else
    {
        Action.Trace("Published Sandbox: " + uniqueId);
    }
}


function CreateFirewallRelations() {
    Action.Trace("******** Create firewall Relations ******");
    for (i = 0; i < adapterInformation.length; i++) {
        if (adapterInformation[i].length == 3) {
            var relationName = firewallName + "- Relation - " + i;
            ret = Action.AddFWRelationRule(relationName, firewallLevel, uniqueId, adapterInformation[i][2], "", adapterInformation[i][0], adapterInformation[i][1]);
            if (ret != 0)
                Action.Trace("Failed Relation: " + relationName + ", friendly name: " + adapterInformation[i][0] + ", mac address: " + adapterInformation[i][1] + ", type: " + adapterInformation[i][2] + " failed: " + ret);
            else
                Action.Trace("Created Relation: " + relationName + ", friendly name: " + adapterInformation[i][0] + ", mac address: " + adapterInformation[i][1] + ", type: " + adapterInformation[i][2]);
        }
        else {
            Action.Trace("Adapter Information: " + i + " , is invalid");
        }
    }
}


// Helper and Display Routines

function DisplayVersion(name)
{
    Action.Trace( name + ": " + Query.Version(name, "Major") + "." + Query.Version(name, "Minor") + "." + Query.Version(name, "Revision") + "." + Query.Version(name, "Build"));
}

function DisplayVersionInfo()
{
    Action.Trace("");
    Action.Trace(" ******** Version Information ********* ");
    Action.Trace("");
    DisplayVersion("Client");
    DisplayVersion("Query");
    DisplayVersion("Action");
    DisplayVersion("Storage");
}


function DisplayTriggerParameters()
{
    Action.Trace("");
    Action.Trace(" ******** Trigger Parameters ********* ");
    Action.Trace("");
    Action.Trace("Reason = " + Query.TriggerParameter("reason"));
    Action.Trace("Parameter Count = " + Query.TriggerParameterCount);
    for(var idx = 0; idx < Query.TriggerParameterCount; idx++)
    {
        Action.Trace("Parameter: " + Query.TriggerParameterName(idx) + " -> " + Query.TriggerParameterValue(idx));
    }
}

function DisplayThisScriptInfo (script)
{
        Action.Trace("");
        Action.Trace(" ******** This Script Information ********* ");
        Action.Trace("Script Name: " + Query.ScriptName);
        Action.Trace("Script ID: " + Query.ScriptID);
        Action.Trace("Script Version: " + Query.ScriptVersion); // Not in Documentation
        Action.Trace("Script Context: " + Query.ScriptContext);
}

// Firewall Helper and Display Routines
function DisplayFWSettings(setting) {
    Action.Trace("        " + setting.Name + " -> " + setting.Value);
}

function DisplayFWPort(port) {
    Action.Trace("        " + port.Type + " : " + port.Behavior + " -> " + port.Start + " - " + port.End);
}

function DisplayFWAcl(acl) {
    var aclString = acl.Behavior + " : " + acl.Type + " -> " + acl.Value;
    if (acl.MacAddress != "") {
        aclString = aclString + "," + acl.MacAddress;
    } 
    Action.Trace("        " + aclString);
    if (acl.Behavior == "port_defined") {
        var ret = acl.GetPortList();
        if (ret.Count > 0) {
            Action.Trace("Ports:");
            for (var i = 0; i < ret.Count; i++) {
                DisplayFWPort(ret.Item(i));
            }
        }
    }
}

function DisplayFirewall(firewall) {
    Action.Trace(" ******** Firewall Info ****** ");
    Action.Trace("Id: " + firewall.Id);
    if (firewall.Name != "")
        Action.Trace("Name: " + firewall.Name);
    Action.Trace("Behavior: " + firewall.Behavior);
    if (firewall.RuleId != "")
        Action.Trace("Rule Id: " + firewall.RuleId);
    if (firewall.RuleName != "")
        Action.Trace("Rule Name: " + firewall.RuleName);

    var ret = firewall.GetStandardControlList(); 
    if (ret.Count > 0) {
        Action.Trace("Standard Control:");
        for (var i = 0; i < ret.Count; i++) {
            DisplayFWSettings(ret.Item(i));
        }
    }

    var ret = firewall.GetStandardPortRuleList();
    if (ret.Count > 0) {
        Action.Trace("Standard Port Rules:");
        for (var i = 0; i < ret.Count; i++) {
            DisplayFWSettings(ret.Item(i));
        }
    }

    var ret = firewall.GetPortRuleList();
    if (ret.Count > 0) {
        Action.Trace("Ports:");
        for (var i = 0; i < ret.Count; i++) {
            DisplayFWPort(ret.Item(i));
        }
    }

    var ret = firewall.GetStandardAclList();
    if (ret.Count > 0) {
        Action.Trace("Standard Acl:");
        for (var i = 0; i < ret.Count; i++) {
            DisplayFWSettings(ret.Item(i));
        }
    }

    var ret = firewall.GetAclList();
    if (ret.Count > 0) {
        Action.Trace("ACLs:");
        for (var i = 0; i < ret.Count; i++) {
            DisplayFWAcl(ret.Item(i));
        }
    }

}

function DisplayFirewalls() {
     // Full list of all firewalls currently defined by the system
     Action.Trace(" ******** List Firewall ****** ");
     var ret = Query.GetFirewallList();  // Not Documentmented
     for (var i = 0; i < ret.Count; i++) {
         DisplayFirewall(ret.Item(i));
     }
 }


 // Adapter Information from the firewall component prospective.
 // Show which firewalls are to be used for each adapter in the system.

 function DisplayFWAdapterInfo(adapter) {
     Action.Trace(" ******** FW Adapter Info ****** ");
     Action.Trace("Adapter: ");
     Action.Trace("   Id: " + adapter.Id);
     Action.Trace("   Name: " + adapter.Name);
     Action.Trace("   Device: " + adapter.Device);
     Action.Trace("   Friendly Name: " + adapter.FriendlyName);
     Action.Trace("   Type: " + adapter.AdapterType);
     Action.Trace("   Current MAC: " + adapter.CurrentMac);
     Action.Trace("   Permanent MAC: " + adapter.PermanentMac);
     Action.Trace("Firewall:");
     Action.Trace("   Id: " + adapter.FirewallId);
     Action.Trace("   Name: " + adapter.FirewallName);
     Action.Trace("Rule:");
     Action.Trace("   Relation Tag: " + adapter.RelationTag);
     Action.Trace("   Id: " + adapter.RuleId);
     Action.Trace("   Name: " + adapter.RuleName);
     Action.Trace("   Level: " + adapter.RuleLevel);
 }

 function DisplayFirewallAdapterInfo() {
     Action.Trace(" ******** List Firewall to Adapter information ****** ");
     var ret = Query.GetFWAdapterList();  // Not Documentmented
     for (var i = 0; i < ret.Count; i++) {
         DisplayFWAdapterInfo(ret.Item(i));
     }
 }


 // Display rules for matching which firewall to which adaapter
 function DisplayFWRelationRule(rule) {
     Action.Trace(" ******** FW Relation Rule ****** ");
     Action.Trace("Tag: " + rule.Tag);
     Action.Trace("Adapter: ");
     if (rule.AdapterType != "any")
        Action.Trace("   Type: " + rule.AdapterType);
     if (rule.AdapterId != "")
        Action.Trace("   Id: " + rule.AdapterId);
     if (rule.FriendlyName != "")
         Action.Trace("   Friendly Name: " + rule.FriendlyName);
     if (rule.MacAddress != "")
         Action.Trace("   Mac Address: " + rule.MacAddress);

     Action.Trace("Firewall:");
     Action.Trace("   Id: " + rule.FirewallId);
     Action.Trace("   Name: " + rule.FirewallName);

     Action.Trace("Producer:");
     Action.Trace("   Name: " + rule.ProducerName);
     Action.Trace("   Feature: " + rule.ProducerFeature);

     Action.Trace("Rule");
     Action.Trace("   Name:  " + rule.RuleName);
     Action.Trace("   Id:    " + rule.RuleId);
     Action.Trace("   Level: " + rule.RuleLevel);
     Action.Trace("   Established: " + rule.Established);
     if (rule.Comment != "")
        Action.Trace("   Comment: " + rule.Comment);
 }

 function DisplayFirewallRelationRules() {
     Action.Trace(" ******** List Firewall Relation Rules ****** ");
     var ret = Query. GetFWRelationRuleList();  // Not Documentmented
     for (var i = 0; i < ret.Count; i++) {
         DisplayFWRelationRule(ret.Item(i));
     }
 }

